/* Bestiole marcheuse par basculement du corp, par les pattes arrières   
et avance des pattes avants. Deux servo permettent le déplacement et 
l'adhérence des pieds avants assure la traction alors que ceux de derrières
suivent en roue libre. Un mode autonomme est possible et l'anti collision est
assurée par un détecteur à ultrasons! Un pilotage manuel peut se faire par
bluetooth et depuis une tablette ou un smart phone ... Le programme vous
laisse la liberté pour toutes modifications ...
B.LECLERC 11/2018
*/

#include <Servo.h> //Déclaration des sous-programmes de composants

#define TRIGG 10 // Désignation des Entrées/Sorties
#define ECHO 9   
#define ledVerte  7
#define ledRouge  8
#define btOk  12


const long TIMEOUT = 25000UL; // Calcul de la relation temps de l'éco du son <-> distance obstacle
float son = 340.0 / 1000;


Servo frontServo; //Désignation des servos
Servo rearServo;

   int centerPos = 85; //Valeur des positions des servos suivant les pas à effectuer 
   int frontRightUp = 120; 
   int frontLeftUp = 50;
   int frontRightTurn = 100; 
   int frontLeftTurn = 70; 
   int backRightForward = 70; 
   int backLeftForward = 100;
   
   char cha; //Déclaration du caractère reçu sur la ligne RS, via le Bluetooth
   
   void moveForward()   {    // Pour faire un pas en avant            
                rearServo.write(backLeftForward);
                delay(700);  // Les delaiys permettent aux servox de rejoindre leur position
                frontServo.write(frontRightUp);
                delay(700);
                rearServo.write(backRightForward);
                delay(700);
                frontServo.write(frontLeftUp);
                delay(700);
               }

  void moveBack()   {     // Pour faire un pas en arrière
                rearServo.write(backLeftForward);
                delay(700);
                frontServo.write(frontLeftUp);
                delay(700);
                rearServo.write(backRightForward);
                delay(700);
                frontServo.write(frontRightUp);
                delay(700);
               }
               
void moveNone()   {          // Position repos 
                rearServo.write(centerPos);
                frontServo.write(centerPos);
                delay(1000);
               }
               

 void turnLeft()   {        // Pour tourner à gauche        
                rearServo.write(backLeftForward);
                delay(700);
                frontServo.write(frontRightTurn);
                delay(700);
                rearServo.write(backRightForward);
                delay(700);
                frontServo.write(frontLeftUp);
                delay(700);
               }

void turnRight()   {        // Pour tourner à droite        
                rearServo.write(backLeftForward);
                delay(700);
                frontServo.write(frontLeftTurn);
                delay(700);
                rearServo.write(backRightForward);
                delay(700);
                frontServo.write(frontLeftUp);
                delay(700);
               }


void setup()   {           //Prédispositions requises avant le dapart des cycles    
                frontServo.attach(3); //Pin où sont raccordé les commandes de servos
                rearServo.attach(5);
                pinMode (ledVerte, OUTPUT);  //Direction des pins
                pinMode (ledRouge, OUTPUT);
                pinMode (TRIGG, OUTPUT);
                digitalWrite (TRIGG, LOW); //Maintient bloquage envoi ultrason
                pinMode (ECHO, INPUT);
                pinMode (btOk, INPUT);
                Serial.begin(9600); //Initialisation de la com série 
               }


void loop()    {                    //Départ du cycles défini par le programme
                 digitalWrite(ledVerte, HIGH) ;  // Led verte allumée
                 digitalWrite(ledRouge, LOW) ;  // Led rouge éteinte
                 moveForward(); //Un pas en avant

                 digitalWrite(TRIGG,HIGH); //Envoi d'un puls d'ultrason de 10ms
                 delayMicroseconds(10);
                 digitalWrite(TRIGG,LOW);

                 int mesure = pulseIn(ECHO,HIGH,TIMEOUT); //Attente de l'écho
                 float distance_mm = mesure /2.0*son;  //Calcul de la distance 

                 if(distance_mm < 200) {         //Si la distance est inférieure à 200mm => Obstacle !
                 digitalWrite(ledRouge, HIGH) ;  //Led rouge allumée
                 digitalWrite(ledVerte, LOW) ;  // Led verte éteinte
                 moveNone();  //Pause marquée
                 delay (1000);
                    for (int i=0; i <= 3; i++){     // Pour deux fois
                      moveBack();       // Un pas en arrière
                    delay(700);
                    turnLeft();  // Tourne à droite
                     } 
                 delay (1000);
                 }

                 if (digitalRead(btOk)) {   // Le récepteur Bluetooth est-il activé ? Si Oui...
                 moveNone();   //Pause marquée
                 digitalWrite(ledVerte, LOW) ; // Led verte éteinte
                 delay (50);
                 goto tlCmd; // Saute au sous programme de marche pilotée
                 }

                 return; //Si non, retour au début (loop )

tlCmd:           
                 Serial.flush();  //Attente d'un caractère
                 cha = Serial.read();  // Lecture du caractère
                 if (Serial.available()>0) { //Valide si plus grand que nul
                 delay (50);
                 

                 if(cha=='A'){  //Si c'est le caractère A
                 digitalWrite(ledVerte, HIGH) ;  // Led verte allumée
                 moveForward();   // un pas en avant
                 digitalWrite(ledVerte, LOW) ; // Led verte éteinte
                 }

                 if(cha=='R'){   //Si c'est le caractère A
                 digitalWrite(ledRouge, HIGH) ;  //Led rouge allumée
                 moveBack();     // etc....
                 digitalWrite(ledRouge, LOW) ; // Led rouge éteinte
                 }

                 if(cha=='G'){
                 digitalWrite(ledRouge, HIGH) ;  //Led rouge allumée
                 digitalWrite(ledVerte, HIGH) ;  // Led verte Allumée
                 turnLeft();
                 digitalWrite(ledVerte, LOW) ;  // Led verte éteinte
                 digitalWrite(ledRouge, LOW) ; // Led rouge éteinte 
                 }

                 if(cha=='D'){
                 digitalWrite(ledRouge, HIGH) ;  //Led rouge allumée
                 digitalWrite(ledVerte, HIGH) ;  // Led verte Allumée
                 turnRight();
                 digitalWrite(ledVerte, LOW) ;  // Led verte éteinte
                 digitalWrite(ledRouge, LOW) ; // Led rouge éteinte 
                 }

                 if(cha=='N'){
                 moveNone(); 
                 }
                 }
                 
                 digitalWrite(ledRouge, LOW) ; // Led rouge éteinte
                 delay (50);                
                 goto tlCmd;  //Et on boucle sur l'étiquette tlCmd
                                 
               }
